#ifndef __FROMWEBAPI_H_
#define __FROMWEBAPI_H_

#include "spdlog/spdlog.h"
#include "WebAPIBase.h"
#include "GlobalVariable.h"
#include <string>


class FromWebAPI : public WebAPIBase
{

public:
    FromWebAPI();
    ~FromWebAPI() override;

    /* 获取数据库中的对比项信息 */
    bool getCompareItemInfo(QList<CompareItemInfo_t>& listItems);
    /* 添加对比项信息 */
    bool insertCompareItem(const QList<CompareItemInfo_t>& listItems);
    /* 插入对比项通道信息 */
    bool insertCompareItemRoad(const QList<CompareItemInfo_t>& listItems);
    /* 只更新对比项信息 */
    bool updateCompareItemOnly(const QList<CompareItemInfo_t>& listItems);
    /* 只更新对比项通道信息，通过先删除所有的itemID的对比项，再插入所有的itemID的对比项来实现 */
    bool updateCompareItemRoadOnly(const QList<CompareItemInfo_t>& listItems);
    /* 更新一条对比项通道信息 */
    bool updateCompareItemOneRoadOnly(const QList<CompareItemInfo_t>& listItems);
    /* 删除对比项信息，同时也会删除对比项通道 */
    bool deleteCompareItem(const QList<int> listIDs);
    /* 删除对比项通道信息 */
    bool deleteCompareItemRoad(const QList<int>& listIDs);
    /* 只删除对比项通道信息，int是对比项ID */
    bool deleteCompareItemRoad(QMap<int, QList<int>>& mapItemRoads);

    /* ================================================================================== */
    /* 获取系统配置信息 */
    bool getSystemConfig(QMap<std::string, std::string>& mapConfig);
    /* 更新系统配置信息  */
    bool updateSystemConfig(const std::string& key, const std::string& value, const std::string& desc = std::string());

    /* ================================================================================== */
    /* 获取检测时段配置 */
    bool getDetectPeriodConfig(QMap<int, DetectPeriodConfig_t>& mapDetectConfig);
    /* 插入检测时段配置 */
    bool insertDetectPeriodConfig(const QMap<int, DetectPeriodConfig_t>& mapDetectConfig);
    /* 删除某些对比项的检测时段 */
    bool deleteDetectPeriodConfig(const QList<int>& listIDs);
    

private:
    std::shared_ptr<spdlog::logger> m_logger = nullptr;

};


#endif // __FROMWEBAPI_H_
