﻿#ifndef ACASERVERSETINFO_H
#define ACASERVERSETINFO_H

#include <QtCore/qglobal.h>
#include <QWidget>

#include "DataBase/entities.h"

#ifdef ACASERVERSETINFO_LIBRARY
#define DLLEXPORT_API Q_DECL_EXPORT
#else
#define DLLEXPORT_API Q_DECL_IMPORT
#endif

//------------------------------------------------------------------------
//函    数: DoInit()
//
//说    明:初始化dll
//
//参    数:
//       pData: 初始化数据
//返 回 值：0:成功; 其他：失败(详见返回码定义)
//------------------------------------------------------------------------
extern "C"
{
   int DLLEXPORT_API DoInit(const stInitData* pData);
}

//------------------------------------------------------------------------
//函    数: DoCreateWindow(int skintype, QWidget* parent)
//
//说    明: 创建窗口
//
//参    数: skintype:皮肤类型 0 白色风格  1 黑色风格
//             parent 父窗口指针（父窗口隐藏子窗口也会隐藏，这里是必填）
//
//返 回 值：0:成功; 其他：失败(详见返回码定义)
//------------------------------------------------------------------------
extern "C"
{
   int DLLEXPORT_API DoCreateWindow(int nSkinType, QWidget* parent);
}

//------------------------------------------------------------------------
//函    数: DoShowWindow(int skintype, int serviceid, bool showWindow)
//
//说    明: 设置窗口显示状态、加载配置信息
//
//参    数: skintype 皮肤类型
// serviceid: 服务id唯一标识，如果小于0界面显示默认值，大于0获取对应的配置数据
// showWindow: true 显示 false 隐藏
//
//返 回 值：0:成功; 其他：失败(详见返回码定义)
//------------------------------------------------------------------------
extern "C"
{
   int DLLEXPORT_API DoShowWindow(int nSkinType, int nServiceID, bool bShowWindow);
}

//------------------------------------------------------------------------
//函    数: DoGetChangedData
//
//说    明: 获取数据修改信息
//
//参    数: data 数据变化信息，没变化返回"", 用来保存日志
//
//返 回 值：0:成功; 其他：失败(详见返回码定义)

//------------------------------------------------------------------------
extern "C"
{
   int DLLEXPORT_API DoGetChangedData(QString& strData);
}

//------------------------------------------------------------------------
//函    数: DoSave(int serviceid)
//
//说    明: 保存数据配置信息
//
//参    数: serviceid 服务id唯一标识
//
//返 回 值：0:成功; 其他：失败(详见返回码定义)

//------------------------------------------------------------------------
extern "C"
{
   int DLLEXPORT_API DoSave(int nServiceID);
}

//------------------------------------------------------------------------
//函    数: DoRelease()
//
//说    明: 释放资源
//
//参    数: void
//
//返 回 值：0:成功; 其他：失败(详见返回码定义)
//------------------------------------------------------------------------
extern "C"
{
   int DLLEXPORT_API DoRelease();
}

#endif // ACASERVERSETINFO_H
